<?php
  // ---------------------------------------------- //
  // Filename: caption_edit.php
  // Version: 1.0
  // Description: Simple album edition
  // Demo: http://album-demo.kwikwi.net/
  // Author: Stephane GULLY <http://www.kwikwi.net>
  // Contact: stephane.gully@libertysurf.fr
  // License: LGPL (http://www.gnu.org/copyleft/lesser.html)
  // ---------------------------------------------- //

  // ---------------------------------------------- //
  // INSTALL
  // ---------------------------------------------- //
  // - Run album with at least these options :
  //      -caption_edit     
  //      -theme Your_Theme
  // - Check that your web serveur has writing 
  //   access to html/txt files and can create new one.
  //   (caption.txt, header.txt, footer.txt, *.html)
  // - Copy caption_edit.php and caption_edit.conf.php
  //   into your album root directory.
  // - Edit caption_edit.conf.php 
  // - Access caption_edit.php with your web browser.
  //   (you need to have php extention enabeled)
  // ---------------------------------------------- //

  // ---------------------------------------------- //
  // WHY ?
  // ---------------------------------------------- //
  // --- FRENCH ---
  // J'ai ecrit ce script car je voulais pouvoir 
  // editer mes albums :
  //    - Sans avoir a effectuer une installation
  //      trop complexe.
  //    - Sans devoir lancer "album" pour regenerer
  //      mes pages html.
  // Il existe un script similaire caption_edit.cgi
  // mais je lui reproche : d'avoir une installation
  // trop complexe (peutetre car je ne suis pas 
  // un adepte des scripts cgi), et de se reposer 
  // sur "album" pour regenerer les pages html
  // alors qu'il pourrait s'en passer. Reposer sur
  // album entraine une installation difficile sur
  // certaines plateformes, MSWindows par exemple.
  // --- ENGLISH ---
  // I've written this script because I wanted to
  // edit my albums :
  //     - With a simple installation.
  //     - Without running "album" each time to
  //       regenerate my html pages.
  // Another similar script (caption_edit.cgi) exists.
  // I don't like to use it because it has a complex
  // installation (I am not a master of cgi scripts),
  // and it uses "album" script to generate
  // the new commented html files. Using "album" to
  // regenerate the html files could cause
  // problems (or hard configuration) on non-unix
  // plateformes : MSWindows for example.
  // ---------------------------------------------- //
  
  // ---------------------------------------------- //
  // CHANGELOG
  // ---------------------------------------------- //
  // + : new feature
  // - : deprecated feature
  // * : bugfix
  // 
  // TODO
  // ------------------------
  // + ajouter un boutton pour regenerer les pages qd 
  //   le mode album est actif (pour le moment c'est 
  //   fait a chaque click sur "Save")
  // + edition de header.txt et footer.txt
  // + edition des commentaires d'un repertoire (caption.txt)
  // + edition des images dans la vue thumbnail.
  // * optimisations de la generation des pages
  // + ameliorer le system de navigation propre 
  //   (generer a la volee des thumbnails dans un cache ?)
  // * debugguer l'edition des captions avec la navigation propre
  //
  // Version 1.00, 2004/08/27
  // ------------------------
  // + caption.txt edition
  // + html regeneration with "album" or builtin 
  // + simple authentification system (login/password)
  // + album navigation system
  // + own navigation system (buggy)
  // ---------------------------------------------- //

  // ---------------------------------------------- //
  // CODE
  // ---------------------------------------------- //
  session_start ();
  session_name("caption_edit.php");

  // setup configuration instance
  $cfg =& Settings::Instance();
  $cfg->setFilename( "caption_edit.conf.php" );
  $LOGIN    = $cfg->get("login");
  $PASSWORD = $cfg->get("password");

  // get page parameters
  $toedit = urldecode($_GET["toedit"]);
  $path   = $_GET["path"] ? urldecode($_GET["path"]) : "./";
  $modify = $_GET["modify"] ? $_GET["modify"] : $_POST["modify"];
  $submit = $_POST["submit"];
  $title   = stripslashes($_POST["title"]);
  $caption = stripslashes($_POST["caption"]);

  // authentification
  if (!authentification( $LOGIN, $PASSWORD ))
  {
    ShowFooter();
    exit;
  }

  // used to identify file types
  $ft =& FileType::Instance();

  // parameters formating
  if (substr($path,-1) != "/")
    $path .= "/";
  // ./azerty/../ => ./
  $path = preg_replace('/(\/)([^\/]*\/)(\.\.\/)/i', '\\1', $path);
  // ./azerty/tn/ => ./azerty/
  $path = preg_replace('/^(.*\/)('.preg_quote($cfg->get("dir")).'\/)(.*\/*)/i', '\\1\\3', $path); // */
  if ($toedit == $cfg->get("index"))
    $toedit = "";
  $toedit_is_html = $ft->isHTML($toedit);
  $toedit_media   = preg_replace('/(.*)(\.'.preg_quote($cfg->get("index")).')/i', '\\1', $toedit);
 
  // Check that requested path is a subdirectory of the current one.
  if ( !path_is_valide($path) )
  {
    ShowHeader();
    echo "<p>ERROR ! You are not allowed to access this path [".$path."]</p>";
    ShowFooter();
    exit;
  }  
  
  // load the directory index file
  $dir_file = new AlbumHtmlFile( $path.$cfg->get("index") );
  // load the caption file
  $caption_file = new AlbumCaptionFile( $path."captions.txt" );
  $media_file     = $toedit ? new AlbumHtmlFile( $path.$cfg->get("dir")."/".$toedit.($toedit_is_html ? "" : ".".$cfg->get("index")) ) : NULL;
  
  // setup the modify/post url 
  $url = $_SERVER["SCRIPT_NAME"];
  $url .= "?path=".urlencode($path);
  $url .= $toedit ? "&toedit=".urlencode($toedit) : "";
  
  if (!$dir_file->isValide())
    ShowHeader();

  // -------------------------------------------------------- //
  $warning_msg = "";
  // verify directory index file
  if (!$dir_file->canOpen())
    $warning_msg .= '<p><strong>Warning ! Directory index file is not readable or don\'t exists. ['.$dir_file->getPathAndFile().']</strong></p>'; 
  else if (!$dir_file->canSave())
    $warning_msg .= '<p><strong>Warning ! Directory index file is not writable or don\'t exists or this directory is not writable. ['.$dir_file->getPathAndFile().']</strong></p>'; 
  else if (!$dir_file->isValide())
    $warning_msg .= '<p><strong>Warning ! Directory index file is not generated by album. ['.$dir_file->getPathAndFile().']</strong></p>'; 
  else if ($toedit_media && !$dir_file->isEditable($toedit_media))
    $warning_msg .= '<p><strong>Warning ! Directory index file is not editable : Missing caption_edit comment tags. Run album (version >=2.47) with -caption_edit option (and a theme). ['.$dir_file->getPathAndFile().']</strong></p>';
  // verify media file
  if ($media_file && !$media_file->canOpen())
    $warning_msg .= '<p><strong>Warning ! Media index file is not readable or dont exists. ['.$media_file->getPathAndFile().']</strong></p>'; 
  else if ($media_file && !$media_file->canSave())
    $warning_msg .= '<p><strong>Warning ! Media index file is not writable or dont exists or this directory is not writable. ['.$media_file->getPathAndFile().']</strong></p>'; 
  else if ($media_file && !$media_file->isValide())
    $warning_msg .= '<p><strong>Warning ! Media index file is not generated by album. ['.$media_file->getPathAndFile().']</strong></p>'; 
  else if ($toedit_media && $media_file && !$media_file->isEditable($toedit_media))
    $warning_msg .= '<p><strong>Warning ! Media index file is not editable : Missing caption_edit comment tags. Run album (version >=2.47) with -caption_edit option (and a theme). ['.$media_file->getPathAndFile().']</strong></p>';
  // verify caption file
  if (!$caption_file->canOpen())
    $warning_msg .= '<p><strong>Warning ! Caption file is not readable or don\'t exists. ['.$caption_file->getPathAndFile().']</strong></p>'; 
  else if (!$caption_file->canSave())
    $warning_msg .= '<p><strong>Warning ! Caption file is not writable or don\'t exists or this directory is not writable. ['.$caption_file->getPathAndFile().']</strong></p>'; 
  // -------------------------------------------------------- //


  // something to save ?
  if ($toedit && $submit == "Save")
  {
    // modify and save caption.txt file
    if (($modify == "caption" || $modify == "both") && $media_file->isCaptionEditable($toedit_media))
    {
      $caption_file->setCaption($toedit_media, $caption);
      $warning_msg .= '<p><strong>Caption written.</strong></p>'; 
    }
    if (($modify == "title" || $modify == "both") && $media_file->isTitleEditable($toedit_media))
    {
      $caption_file->setTitle($toedit_media, $title);
      $warning_msg .= '<p><strong>Title written.</strong></p>'; 
    }
    $caption_file->save();

    if ($cfg->get("html_gen") == "album")
    {
      // execute the album script to regenerate html files
      $album_cmd  = $cfg->get("album");
      $album_cmd .= " -depth 1";
      $album_cmd .= $cfg->get("album.conf") ? " -conf ".$cfg->get("album.conf") : "";      
      exec('cd '.$path.' && '.$album_cmd);
    }
    else
    {
      // modify and save the html files without using album script
      if (($modify == "caption" || $modify == "both") && $media_file->isCaptionEditable($toedit_media))
      {
        $dir_file->setCaption($toedit_media, $caption);
        $media_file->setCaption($toedit_media, $caption);
      }
      if (($modify == "title" || $modify == "both") && $media_file->isTitleEditable($toedit_media))
      {
        $dir_file->setTitle($toedit_media, $title);
        $media_file->setTitle($toedit_media, $title);
      }
      $dir_file->save();
      $media_file->save();
    }
    $modify = "";
  }

  // own navigation
  if (!$dir_file->isValide())
  {
    // quelque chose a editer ? une image ou un repertoire ?
    // on affiche l'image et un formulaire d'edition
      
    if ($toedit && !$toedit_is_html)
    {
      echo "<p><img alt=\"".$toedit."\" src=\"".$path.$toedit."\"/></a></p>";
    }

    // todo : clean the formulary creation => use the buildForm function
    if ($toedit)
    {
      // show the formulary
      if ($modify)
        echo '<form method="post" action="'.$url.'">';
      echo '<table cellpadding="10" border="0" summary="Caption and title edition">';
      if ($modify == "title")
      {
        $title = $caption_file->getTitle($toedit_media, false);
        echo '  <tr>';
        echo '    <td><label for="title">Title</label></td>';
        echo '    <td><input type="text" size="80" maxlength="2000" name="title" value="'.$title.'" id="title"/></td>';
        echo '  </tr>';
      }
      else
      {
        $title = $caption_file->getTitle($toedit_media, true);
        echo '  <tr>';
        echo '    <td>Title</td>';
        echo '    <td>'.$title.'<td>';
        echo '    <td><a href="'.$url."&modify=title".'">[Modify]</a><td>';
        echo '  </tr>';
      }
      if ($modify == "caption")
      {
        $caption = $caption_file->getCaption($toedit_media, false);
        echo '  <tr>';
        echo '    <td><label for="caption">Caption</label></td>';
        echo '    <td><textarea rows="6" cols="64" name="caption" id="caption">'.$caption.'</textarea></td>';
        echo '  </tr>';
      }
      else
      {
        $caption = $caption_file->getCaption($toedit_media, true);
        echo '  <tr>';
        echo '    <td>Caption</td>';
        echo '    <td>'.$caption.'<td>';
        echo '    <td><a href="'.$url."&modify=caption".'">[Modify]</a><td>';
        echo '  </tr>';
      }
      echo '</table>';
      if ($modify)
      {
        echo '<input type="hidden" name="modify" value="'.$modify.'"/>';
        echo '<input type="submit" name="submit" value="Save"/>';
        echo '</form>';
      }
    }
    
    // show own navigation
    echo $warning_msg;
    browse_directory($path);
    ShowFooter();    
  }
  else
  {
    // show the navigation page
    if (!$toedit_is_html)
    {
      // this is the directory browsing
      $form =& buildForm( $toedit_media, $url, &$caption_file, $modify, 20, "WITHOUT_MODIFY" );
      // directory navigation
      $dir_file->show( &$form, $warning_msg );
    }
    else
    {
      // this is the media file edition
      $form =& buildForm( $toedit_media, $url, &$caption_file, $modify, 60, "WITHOUT_MODIFY" );
      // file by file navigation
      $media_file->show( &$form, $warning_msg ); 
    }
  }

  
  
  /* -------------------------------------------------------- */
  /* - Classes and functions -------------------------------- */
  /* -------------------------------------------------------- */
  
  /*
   * Settings
   */  
  class Settings
  {
    var $options = array();
    var $filename = "";
    
    function & Instance()
    {
      static $m_Instance;
      if (!isset($m_Instance))
        $m_Instance = new Settings();
      return $m_Instance;
    }
    
    function Settings()
    {
      $this->options["index"]    = "index.html";
	    $this->options["dir"]      = "tn";
      $this->options["login"]    = "guest";
      $this->options["password"] = "guest";
    }

    function setFilename( $filename = "caption_edit.conf.php" )
    {
      $this->filename = $filename;
      
      // load the config file if it exists
      if (file_exists($this->filename))
	    {
  	    include($this->filename);
        reset($options);
        foreach($options as $key => $value)
          $this->options[$key] = $value;
      }
      
      // load the album.conf options
      if ($this->options["album.conf"] && file_exists($this->options["album.conf"]))
      {
        $lines = @file($this->options["album.conf"]);
        reset($lines);
        foreach($lines as $key => $value)
        {
          if ($value == "\n")
            continue;
          
          $pattern = '/(index)(\ *)(.*)(.|\n)/i';
          if (preg_match($pattern,$value))
            $this->options["index"] = preg_replace($pattern, "\${3}", $value);

          $pattern = '/(dir)(\ *)(.*)(.|\n)/i';
          if (preg_match($pattern,$value))
            $this->options["dir"] = preg_replace($pattern, "\${3}", $value);
        } 
      }
      
//      print_r($this->options);
    }
    
    function get($key)
    {
	  if ($key == "version")
        $this->options["version"] = "v1.0";
      return $this->options[$key];
    }

    function set($key, $value)
    {
      $this->options[$key] = $value;
    }
  }

  /*
   * FSDirectory
   */  
  class FSDirectory
  {
    var $path = "./";
    var $dirs  = array();
    var $files = array();
   
    function FSDirectory($path)
    {
      $this->path = $path;
      $this->listCurrentDir();
    }
  
    function listCurrentDir()
    {
      $hndl = opendir($this->path);
      while($file = readdir($hndl))
      {
        if ($file == '.' || $file=='..')
          continue;
        $completepath="$basedir$file";
        if (is_dir($completepath))
        {
          $this->dirs[] = $file;
        }
     	  else
        {
          $this->files[] = $file;
        }
      }
    }
  
    function getFiles( $filter_type = "ALL" )
    {
      if ($filter_type == "ALL")
        return $this->files;
      
    	$ft =& FileType::Instance();
    	$result = array();
      reset($this->files);
      foreach($this->files as $key => $value)
      {
    	  if ($filter_type == "IMG")
          if ($ft->isImage($value))
            $result[] = $value;
    	  else if ($filter_type == "VID")
          if ($ft->isVideo($value))
            $result[] = $value;
    	  else if ($filter_type == "HTM")
          if ($ft->isHTML($value))
            $result[] = $value;
      }
      return $result;
    }
  
    function getDirs()
    {
      return $this->dirs;
    }
  }
  
  
  /*
   * FileType
   */
  class FileType
  {
    var $filter_all   = "";
    var $filter_image = "";
    var $filter_video = "";
    var $filter_html  = "";
   
    function & Instance()
    {
      static $m_Instance;
      if (!isset($m_Instance))
        $m_Instance = new FileType();
      return $m_Instance;
    }
  						
    function FileType()
    {
      $this->filter_all = "^.*$";
      $this->filter_image = "^.*\.(";
      $this->filter_image .= "[jJ][pP][gG]|";
      $this->filter_image .= "[jJ][pP][eE][gG]|";
      $this->filter_image .= "[pP][nN][gG]|";
      $this->filter_image .= "[gG][iI][fF]|";
      $this->filter_image .= "[tT][iI][fF]";
      $this->filter_image .= "[tT][iI][fF][fF]";
      $this->filter_image .= ")$";
      $this->filter_video = "^.*\.([aA][vV][iI]|[mM][oO][vV])$";
      $this->filter_html  = "^.*\.(html|htm)$";
    }
  
    function isHTML($filename)
    {
      return ereg($this->filter_html, $filename);
    }
    
    function isImage($filename)
    {
      return ereg($this->filter_image, $filename);
    }
    
    function isVideo($filename)
    {
      return ereg($this->filter_video, $filename);
    }
  }
  
  /*
   * AlbumCaptionFile
   */
  class AlbumCaptionFile 
  {
    var $caption_filename = "";
    var $contents = array();
    var $cache = array();
    
    function AlbumCaptionFile( $caption_filename )
    {
      $this->caption_filename = $caption_filename;
      $this->load();
    }
   
    function getPathAndFile()
    {
      return $this->caption_filename;
    }
    
    function load()
    {
      $this->contents = array();
      if (file_exists($this->caption_filename))
      {
        $lines = @file($this->caption_filename);
        reset($lines);
      	foreach($lines as $key => $value)
      	{
      	  if ($value == "\n")
      	    continue;
      	  if (strpos($value,"\t"))
      	    $this->contents[] = explode("\t",$value);
      	  else
      	    $this->contents[] = explode("::",$value);
      	}
      }
      
      // check that there is no other images in the directory
      $caption_dirpath = preg_replace("/(.*\/)([^\/]*)/i", "\${1}", $this->caption_filename);
      $fs = new FSDirectory($caption_dirpath);
      $images = $fs->getFiles("IMG");
      reset($images);
      foreach($images as $i_key => $i_value)
      {
        $found = false;
        reset($this->contents);
        foreach($this->contents as $c_key => $c_value)
        {
          if ($c_value[0] == $i_value )
          {
            $found = true;
            break;
          }
        }
        // if a new images has been found, add it to the caption.txt file
        if (!$found)
          $this->contents[] = array($i_value, _nl2br(htmlentities(preg_replace("/(.*)(\..{3})/i", "\${1}", $i_value))), " ");
      }
      
    	return true;
    }
    
    function save()
    {
      if (!$this->canSave())
  	  return false;
    	$to_save = "";
    	reset($this->contents);
    	foreach($this->contents as $key => $value)
      {
    	  $to_save .= $value[0] ? implode("\t", $value) : "";
    	  // add a br if it is not allready present
    	  if (substr($to_save, -1) != "\n")
    	    $to_save .= "\n";
    	}
    	$fp = fopen($this->caption_filename,"w");
    	fwrite($fp, $to_save);
    	fclose($fp);
    	return true;
    }
    
    function canOpen()
    {
      return is_readable ( $this->caption_filename ); 
    }
    
    function canSave()
    {
      if (!file_exists($this->caption_filename))
    	{
        $fp = @fopen($this->caption_filename,"w");
        if ($fp) fclose($fp);
    	}
    	return is_writable ( $this->caption_filename );
    }
    
    function getTitleAndCaption( $filename, $mode_html = false )
    {
      if (isset($this->cache[$filename]))
        return $this->cache[$filename];

      $result["title"]   = "";
      $result["caption"] = "";
  	
      reset($this->contents);
      foreach($this->contents as $key => $value)
      {
    	  if ($value[0] == $filename )
    	  {
    	    // clean possible \n
          $value[1] = str_replace("\n", "", $value[1]);
    	    $value[2] = str_replace("\n", "", $value[2]);

    	    $result["title"]   = $mode_html ? $value[1] : html_entity_decode(_br2nl($value[1]));
    	    $result["caption"] = $mode_html ? $value[2] : html_entity_decode(_br2nl($value[2]));
    		  break;
    	  }
    	}
      $this->cache[$filename] = $result;
    	return $result;
    }
  
    function getTitle($filename, $mode_html = false)
    {
      $result = $this->getTitleAndCaption($filename, $mode_html);
      return $result["title"];
    }
    
    function getCaption($filename, $mode_html = false)
    {
      $result = $this->getTitleAndCaption($filename, $mode_html);
      return $result["caption"];
    }
  
    function setTitle($filename, $new_title)
    {
      $new_title = _nl2br(htmlentities($new_title));
      
      if ($new_title == "")
  	    $new_title = " ";
      
      reset($this->contents);
      foreach($this->contents as $key => $value)
      {
    	  if ($value[0] == $filename )
    	  {
    	    $this->contents[$key][1] = _nl2br($new_title);
    		  return;
    	  }
      }
      $this->contents[] = array( $filename, _nl2br($new_title), " ");
      
      // cache update
      $this->cache[$filename]["title"] = $new_title;
    }
  
    function setCaption($filename, $new_caption)
    {
      $new_caption = _nl2br(htmlentities($new_caption));
       
      if ($new_caption == "")
  	  $new_caption = " ";
  	  
      reset($this->contents);
      foreach($this->contents as $key => $value)
      {
    	  if ($value[0] == $filename )
    	  {
    	    $this->contents[$key][2] = _nl2br($new_caption);
    		  return;
    	  }
      }
      $this->contents[] = array( $filename, " ", _nl2br($new_caption));

      // cache update
      $this->cache[$filename]["caption"] = $new_caption;
    }
  }
  
  /*
   * AlbumHtmlFile
   */
  class AlbumHtmlFile
  {
    var $index_filename = "";
    var $raw_content = "";
    
    function AlbumHtmlFile( $index_filename )
    {
      $this->index_filename = $index_filename;
      $this->load();
    }
    
    function getPathAndFile()
    {
      return $this->index_filename;
    }
    
    function load()
    {
      if (!$this->canOpen() || filesize($this->index_filename) <= 0)
        return false;
    	$fp = fopen($this->index_filename, "r");
    	$this->raw_content = fread($fp, filesize($this->index_filename));
    	fclose($fp);
    	return true;
    }
    
    function save()
    {
      if (!$this->canSave())
        return false;
    	$fp = fopen($this->index_filename, "w");
    	fwrite($fp, $this->raw_content);
    	fclose($fp);
    	return true;
    }
    
    function isValide()
    {
      if (!$this->canOpen() || !$this->canSave())
        return false;
  
      $filter_generated_tag = "^.*<meta\ *name='Generator'\ *content='album\ http://MarginalHacks.com/'\ */>.*$";
      if (!ereg( $filter_generated_tag, $this->raw_content ))
        return false;
  
      return true;
    }
  
    function isEditable( $filename )
    {
      if (!$this->isValide())
        return false;

      if (!$this->isTitleEditable($filename) || !$this->isCaptionEditable($filename))
        return false;
        
      return true;
    }
   
    function isTitleEditable( $filename )
    {
      if (!$this->isValide())
        return false;
         
      $pattern = '/<!--IMAGE_NAME\ *name="NAME[0123456789]{1,4}:'.preg_quote($filename).'"\ *value=".*"-->.*<!--END_IMAGE_NAME-->/s';
      if (!preg_match( $pattern, $this->raw_content))
        return false;
  
      return true;
    }

    function isCaptionEditable( $filename )
    {
      if (!$this->isValide())
        return false;
         
      $pattern = '/<!--IMAGE_CAPTION\ *name="CAPTION:'.preg_quote($filename).'"-->.*<!--END_IMAGE_CAPTION-->/s';
      if (!preg_match( $pattern, $this->raw_content))
        return false;
  
      return true;
    }
    
    function canOpen()
    {
      return is_readable ( $this->index_filename ); 
    }
    
    function canSave()
    {
      if (!file_exists($this->index_filename))
    	{
          $fp = @fopen($this->index_filename,"w");
    	  if ($fp) fclose($fp);
    	}
    	return is_writable ( $this->index_filename );
    }
    
    function setTitle($filename, $new_title)
    {
      $new_title = _nl2br(htmlentities($new_title));

      // escape regex chars
    	$filename = preg_quote($filename,"/");
      
      $pattern = '/(<!--IMAGE_NAME\ *name="NAME[0123456789]{1,4}:'.$filename.'"\ *value=".*"-->)(.*)(<!--END_IMAGE_NAME-->)/';
      $replacement = "\${1}".$new_title."\${3}";
      $this->raw_content = preg_replace($pattern, $replacement, $this->raw_content);
    }
  
    function setCaption($filename, $new_caption)
    {
      $new_caption = _nl2br(htmlentities($new_caption));
      
      // escape regex chars
      $filename = preg_quote($filename,"/");
      
      $pattern = '/(.*<!--IMAGE_CAPTION\ *name="CAPTION:'.$filename.'"-->)(\n*.*)(<!--END_IMAGE_CAPTION-->.*)/i';
      $replacement = "\${1}".$new_caption."\${3}";
      $this->raw_content = preg_replace($pattern, $replacement, $this->raw_content);
    }
    
    function show( &$form, $warning_msg = "" )
    {
      $cfg =& Settings::Instance();
      
    	$result    = $this->raw_content;
      $to_append  = preg_replace("/(.*\/)([^\/]*)/i", "\${1}", $this->index_filename);
      $media_name = preg_replace("/(.*\/)([^\/]*)(\.".preg_quote($cfg->get("index")).")/i", "\${2}", $this->index_filename);
      $media_name = preg_replace("/(.*\/)(".preg_quote($cfg->get("index")).")/i", "", $media_name);

      // ---- Prepare the lines : add \n ---- //
      // to be sure that a \n exist between each <a href=...
      $pattern = "/(<a)/i";
      $replacement = "\n\${1}";
      $result = preg_replace($pattern, $replacement, $result);
      
      // insert a \n before each images because the regex replace doesent work
      // when several patterns match on the same line
      $pattern = "/(<img)/i";
      $replacement = "\n\${1}";
      $result = preg_replace($pattern, $replacement, $result);


      // ---- Do the real work ---- //
      // update the images sources
      $pattern = "/(<img.*src=['\"])(.*)(['\"])/i";
      $replacement = "\${1}".$to_append."\${2}\${3}";
      $result = preg_replace($pattern, $replacement, $result);

      // update the background link
      $pattern = "/(<body.*background=['\"])([^']*)(['\"])/i";
      $replacement = "\${1}".$to_append."\${2}\${3}";
      $result = preg_replace($pattern, $replacement, $result);

      // update generic links 
      $pattern = "/(<a.*href=['\"])(.*\/)([^\/']*".preg_quote($cfg->get("index")).")(['\"][^>]*>)/i";
      $replacement = "\${1}".$_SERVER["SCRIPT_NAME"]."?path=".$to_append."\${2}"."&toedit=\${3}\${4}";
      $result = preg_replace($pattern, $replacement, $result);

      // update CSS links
      // <link REL="stylesheet" TYPE="text/css" href=
      $pattern = "/(<link.*href=['\"])(.*)(['\"][^>]*>)/i";
      $replacement = "\${1}".$to_append."\${2}\${3}";
      $result = preg_replace($pattern, $replacement, $result);

      // update CSS rulez
      // background-image: url("../Themes/NewFloat/h1.gif");
      $pattern = '/(background-image:\ +url\(")(.*)("\)\ *;)/i';
      $replacement = "\${1}".$to_append."\${2}\${3}";
      $result = preg_replace($pattern, $replacement, $result);

      // update navigation links 
      //<a href='105_0533.JPG.index.html'>
      $pattern = "/(<a.*href=['\"])([^?&\/']*".preg_quote($cfg->get("index")).")(['\"][^>]*>)/i";
      $replacement = "\${1}".$_SERVER["SCRIPT_NAME"]."?path=".$to_append."&toedit=\${2}\${3}\n";
      $result = preg_replace($pattern, $replacement, $result);

      if ($media_name)
      {
        // replace IMAGE_NAME field with formulary
        $pattern = '/(<!--IMAGE_NAME\ *name="NAME[0123456789]{1,4}:'.preg_quote($media_name,"/").'"\ *value=".*"-->)(.*)(<!--END_IMAGE_NAME-->)/i';
        $replacement = "\${1}".$form["title"]."\${3}";
        $result = preg_replace($pattern, $replacement, $result);
  
        // remove the link around IMAGE_NAME field
        $pattern = '/(<a href=["\'].*["\']>)([^<>]*)(<!--IMAGE_NAME\ *name="NAME[0123456789]{1,4}:'.preg_quote($media_name,"/").'")(.*)(<!--END_IMAGE_NAME-->)(<\/a>)/i';
        $replacement = "\${2}\${3}\${4}\${5}";
        $result = preg_replace($pattern, $replacement, $result);

        // remove the \n field in front of caption
        $pattern = '/(<!--IMAGE_CAPTION.*-->)(\n)(.*)(<!--END_IMAGE_CAPTION-->)/i';
        $replacement = "\${1}\${3}\${4}";
        $result = preg_replace($pattern, $replacement, $result);

        // replace IMAGE_CAPTION field with formulary
        $pattern = '/(<!--IMAGE_CAPTION\ *name="CAPTION:'.preg_quote($media_name,"/").'"-->)(.*)(<!--END_IMAGE_CAPTION-->)/';
        $replacement = "\${1}".$form["caption"]."\${4}";
        $result = preg_replace($pattern, $replacement, $result);
      }
      // insert the formulary tags
      if ($media_name && $form["begin"] && $form["end"])
      {
        $pattern = '/(<body.*>)/i';
        $replacement = "\${1}".$form["begin"];
        $result = preg_replace($pattern, $replacement, $result);
        
        $pattern = '/(<\/body>)/i';
        $replacement = $form["end"]."\${1}";
        $result = preg_replace($pattern, $replacement, $result);
      }
      
      // insert warning message
      $pattern = '/(<body.*>)/i';
      $replacement = "\${1}".$warning_msg;
      $result = preg_replace($pattern, $replacement, $result);

      // insert caption_edit.php creator info
      $pattern = '/(<\/body>)/i';
      $replacement = "<p style=\"clear: both;\">Edition done by caption_edit.php ".$cfg->get("version")." (by <a href=\"http://www.kwikwi.net\">Stphane GULLY</a>)</p>"."\${1}";
      $result = preg_replace($pattern, $replacement, $result);


      // ----- Cleanup the \n ----- //
      // remove the previously inserted \n for each links
      $pattern = "/(\n<a)/i";
      $replacement = "<a";
      $result = preg_replace($pattern, $replacement, $result);

      // remove the previously inserted \n for each images
      $pattern = "/(\n<img)/";
      $replacement = "<img";
      $result = preg_replace($pattern, $replacement, $result);
      $result = preg_replace($pattern, $replacement, $result);

    	echo $result;
    }
  }
  

  function & buildForm( $media_name, $url, &$caption_file, $modify, $width = 64, $mode = "WITH_MODIFY")
  {
    $form = array();
    
    if ($mode == "WITH_MODIFY")
    {
      // build the formulary
      $form["begin"] .= '<form method="post" action="'.$url.'">';
      $form["end"] .= '<input type="hidden" name="modify" value="'.$modify.'"/>';
      $form["end"] .= '<input type="submit" name="submit" value="Save"/>';
      $form["end"] .= '</form>';
      
      if ($modify == "title")
      {
        $title = $caption_file->getTitle($media_name, false);
        $form["title"] .= '<input type="text" size="80" maxlength="2000" name="title" value="'.$title.'" id="title"/>'; 
      }
      else
      {
        $title = $caption_file->getTitle($media_name, true);
        $form["title"] .= $title; 
        $form["title"] .= ' <a href="'.$url."&modify=title".'">[Modify]</a>';
      }
      if ($modify == "caption")
      {
        $caption = $caption_file->getCaption($media_name, false);
        $form["caption"] .= '<textarea rows="6" cols="64" name="caption" id="caption">'.$caption.'</textarea>'; 
      }
      else
      {
        $caption = $caption_file->getCaption($media_name, true);
        $form["caption"] .= $caption; 
        $form["caption"] .= ' <a href="'.$url."&modify=caption".'">[Modify]</a>'; 
      }
    }
    else if ($mode == "WITHOUT_MODIFY")
    {
      $title = $caption_file->getTitle($media_name, false);
      $form["title"] .= '<input type="text" size="'.$width.'" maxlength="200" name="title" value="'.$title.'" id="title"/>';
      $form["title"] .= '<input type="submit" name="submit" value="Save"/>';

      $caption = $caption_file->getCaption($media_name, false);
      $form["caption"] .= '<textarea rows="6" cols="'.$width.'" maxlength="2000" name="caption" id="caption">'.$caption.'</textarea>'; 
      $form["caption"] .= '<input type="submit" name="submit" value="Save"/>';
      
      $form["begin"] .= '<form method="post" action="'.$url.'">';
      $form["end"] .= '<input type="hidden" name="modify" value="both"/>';
      $form["end"] .= '</form>';
    }
    
    return $form;
  }
  
  
  function ShowHeader()
  {
    echo '<html>';
  }
  
  function ShowFooter()
  {
    echo '</html>';
  }
  
  function browse_directory( $basedir )
  {
    $filter_image = "^.*\.(";
    $filter_image .= "[jJ][pP][gG]|";
    $filter_image .= "[jJ][pP][eE][gG]|";
    $filter_image .= "[pP][nN][gG]|";
    $filter_image .= "[gG][iI][fF]|";
    $filter_image .= "[tT][iI][fF]";
    $filter_image .= "[tT][iI][fF][fF]";
    $filter_image .= ")$";
    $filter_video = "^.*\.([aA][vV][iI]|[mM][oO][vV])$";
    $filter_html  = "^.*\.(html|htm)$";
  
    $dir_tree = array();
    build_dirtree( $basedir, &$dir_tree );
   
    // show directories
    reset($dir_tree["dirs"]);
    echo "<ul>";
    // show back
    $back_path = urlencode($basedir."../");
    if ($back_path)
    {
      $url = $_SERVER["SCRIPT_NAME"];
      $url .= "?path=".$back_path;
      echo "<li><a href=\"".$url."\">".".."."</a></li>";
    }
    foreach($dir_tree["dirs"] as $key => $value)
    {
      $url = $_SERVER["SCRIPT_NAME"];
      $url .= "?path=".urlencode($basedir.$key."/");
      echo "<li><a href=\"".$url."\">".$key."</a></li>";
    }
    echo "</ul>";
    
    // show files
    $caption_file = new AlbumCaptionFile( $basedir."/captions.txt" );
    reset($dir_tree["files"]);
    foreach($dir_tree["files"] as $key => $value)
    {
    	if (ereg( $filter_image, $value))
    	{
    	  $url = $_SERVER["SCRIPT_NAME"];
    	  $url .= "?path=".$_GET["path"];
    	  $url .= "&toedit=".urlencode($value);
          
    	  $title   = $caption_file->getTitle($value, true);
    	  $caption = $caption_file->getCaption($value, true);
    
    	  echo '<div class="legende">';
    	  echo '<a href="'.$url.'"><img alt="'.$title.'" src="'.$basedir.$value.'"/></a>';
    	  echo '<h2>'.$title.'</h2>';
    	  echo '<p>'.$caption.'</p>';
    	  echo "</div>";
      }
    }
  }
  	
  function build_dirtree( $basedir, &$result )
  {
    $result["dirs"]  = array();
    $result["files"] = array();
    $hndl=opendir($basedir);
    while($file=readdir($hndl))
    {
      if ($file == '.' || $file=='..')
  	  continue;
      $completepath="$basedir$file";
      if (is_dir($completepath))
  	  {
        // its a dir, recurse.
        $result["dirs"][$file] = array();
        build_dirtree($basedir.$file, &$result["dirs"][$file]);
      }
     	else
      {
        // its a file.
        $result["files"][] = $file;
      }
    }
  }
  
  function _nl2br( $data )
  {
    return str_replace("\n", "<br />",str_replace("\r\n", "<br />", $data));
  }
  
  function _br2nl( $data )
  {
    return preg_replace( '!<br.*>!iU', "\n", $data );
  }
  
  // Reverse search of strrchr.
  function strrrchr($haystack,$needle)
  {
    return substr($haystack,0,strrpos($haystack,$needle)+1);
  }
  
  function write_ini_file($path, $assoc_array)
  {
    foreach ($assoc_array as $key => $item)
    {
      if (is_array($item))
      {
        $content .= "\n[$key]\n";
        foreach ($item as $key2 => $item2)
        {
          $content .= "$key2 = \"$item2\"\n";
        }       
      }
      else
      {
        $content .= "$key = \"$item\"\n";
      }
    }

    if (!$handle = fopen($path, 'w'))
      return false;

    if (!fwrite($handle, $content))
      return false;

    fclose($handle);
    return true;
  }

  function path_is_valide($path)
  {
    // check that the requested path is a subdirectory of the current one
    $current_path = getcwd();
    chdir($path);
    $new_path     = getcwd();
    chdir($current_path);

  	// using === is important because when $new_path is valide
  	// strpos returns 0 (0 == false is true but  0 === false is false)
  	if ( strpos( $new_path, $current_path ) === false )
      return false;
  	else
  	  return true;
  }
  
  function authentification( $_user, $_pass )
  {
    if (!isset($_SESSION["logged_in"]))
    {
      $_SESSION["logged_in"] = 0;
      $_SESSION["secure_id"] = 0;
    }
    if (!strcmp($_POST["user"], $_user) && !strcmp($_POST["pass"], $_pass))
    {
      $_SESSION["logged_in"] = 1;
      $_SESSION["secure_id"] = md5(getcwd());
    }

    if ($_GET["logout"] == 1)
    {
      $_SESSION["logged_in"] = 0;
      $_SESSION["secure_id"] = 0;      
    }
    
    if ( $_SESSION["logged_in"] == 1 &&
         $_SESSION["secure_id"] == md5(getcwd()) )
    {
      return true;
    }
    else
    {
      ShowHeader();
      echo '<form action="'.$_SERVER["SCRIPT_NAME"].'" method="post">';
      if ($_user == "guest" && $_pass == "guest")
      {
        echo '<p>for demo: login=guest pass=guest</p>';
        echo 'User: <input type="text" name="user" value="'.$_user.'" /><br/>';
        echo 'Password: <input type="password" name="pass" value="'.$_pass.'" /><br/>';
      }
      else
      {
        echo 'User: <input type="text" name="user" /><br/>';
        echo 'Password: <input type="password" name="pass" /><br/>';
      }      echo '<input type="submit" />';
      echo '</form>';
      return false;
    }
  }
  
  
?>
